/* 
 * Copyright (C) 2013 Olaf Lessenich.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */
package de.fosd.jdime.merge;

import de.fosd.jdime.common.Artifact;
import de.fosd.jdime.common.MergeContext;
import de.fosd.jdime.common.MergeType;
import de.fosd.jdime.common.operations.MergeOperation;
import java.io.IOException;

/**
 * Interface for the merge strategies.
 *
 * @author Olaf Lessenich
 *
 * @param <T> type of artifact
 *
 */
public interface MergeInterface<T extends Artifact<T>> {

    /**
     * At least two input files are needed.
     */
    int MINFILES = MergeType.MINFILES;
    /**
     * More than three input files are not supported at the moment.
     */
    int MAXFILES = MergeType.MAXFILES;

    /**
     * Performs a merge.
     *
     * @param operation merge operation
     * @param context merge context
     * @throws IOException
     * @throws InterruptedException
     */
    void merge(MergeOperation<T> operation, MergeContext context)
            throws IOException, InterruptedException;
}
